@@ -26,29 +26,28 @@
 #include "gtksourcegutterlines-private.h"
 #include "gtksourcesignalgroup-private.h"
 #include "gtksourcestylescheme-private.h"
-#include "gtksourceview.h"
+#include "gtksourceview-private.h"
 #include "gtksourcegutterrenderer.h"
 #include "gtksourcegutterrenderer-private.h"
 
 /**
- * SECTION:gutter
- * @Short_description: Gutter object for GtkSourceView
- * @Title: GtkSourceGutter
- * @See_also: #GtkSourceView, #GtkSourceMark
+ * GtkSourceGutter:
  *
- * The #GtkSourceGutter object represents the left or right gutter of the text
- * view. It is used by #GtkSourceView to draw the line numbers and
- * #GtkSourceMark<!-- -->s that might be present on a line. By packing
- * additional #GtkSourceGutterRenderer objects in the gutter, you can extend the
+ * Gutter object for [class@View].
+ *
+ * The `GtkSourceGutter` object represents the left or right gutter of the text
+ * view. It is used by [class@View] to draw the line numbers and
+ * [class@Mark]s that might be present on a line. By packing
+ * additional [class@GutterRenderer] objects in the gutter, you can extend the
  * gutter with your own custom drawings.
  *
- * To get a #GtkSourceGutter, use the gtk_source_view_get_gutter() function.
+ * To get a `GtkSourceGutter`, use the [method@View.get_gutter] function.
  *
- * The gutter works very much the same way as cells rendered in a #GtkTreeView.
+ * The gutter works very much the same way as cells rendered in a [class@Gtk.TreeView].
  * The concept is similar, with the exception that the gutter does not have an
- * underlying #GtkTreeModel. The builtin line number renderer is at position
- * #GTK_SOURCE_VIEW_GUTTER_POSITION_LINES (-30) and the marks renderer is at
- * #GTK_SOURCE_VIEW_GUTTER_POSITION_MARKS (-20). The gutter sorts the renderers
+ * underlying [iface@Gtk.TreeModel]. The builtin line number renderer is at position
+ * %GTK_SOURCE_VIEW_GUTTER_POSITION_LINES (-30) and the marks renderer is at
+ * %GTK_SOURCE_VIEW_GUTTER_POSITION_MARKS (-20). The gutter sorts the renderers
  * in ascending order, from left to right. So the marks are displayed on the
  * right of the line numbers.
  */
@@ -515,8 +514,6 @@ _gtk_source_gutter_new (GtkTextWindowType  type,
  * @gutter: a #GtkSourceGutter.
  *
  * Returns: (transfer none): the associated #GtkSourceView.
- *
- * Since: 3.24
  */
 GtkSourceView *
 gtk_source_gutter_get_view (GtkSourceGutter *gutter)
@@ -537,9 +534,6 @@ gtk_source_gutter_get_view (GtkSourceGutter *gutter)
  * @renderer cannot be already inserted to another gutter.
  *
  * Returns: %TRUE if operation succeeded. Otherwise %FALSE.
- *
- * Since: 3.0
- *
  **/
 gboolean
 gtk_source_gutter_insert (GtkSourceGutter         *gutter,
@@ -637,8 +631,6 @@ gtk_source_gutter_remove (GtkSourceGutter         *gutter,
  * @position: the new renderer position.
  *
  * Reorders @renderer in @gutter to new @position.
- *
- * Since: 2.8
  */
 void
 gtk_source_gutter_reorder (GtkSourceGutter         *gutter,
@@ -706,7 +698,6 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 {
 	GtkSourceGutter *gutter = GTK_SOURCE_GUTTER (widget);
 	GtkTextView *text_view = GTK_TEXT_VIEW (gutter->view);
-	GtkStyleContext *style_context;
 	const GList *list;
 	GdkRectangle visible_rect;
 	GtkTextIter begin;
@@ -721,8 +712,6 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 		return;
 	}
 
-	style_context = gtk_widget_get_style_context (GTK_WIDGET (gutter));
-
 	gtk_text_view_get_visible_rect (text_view, &visible_rect);
 	gtk_text_view_get_iter_at_location (text_view, &begin,
 	                                    visible_rect.x, visible_rect.y);
@@ -752,6 +741,7 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 		if (cursor_line >= gtk_source_gutter_lines_get_first (gutter->lines) &&
 		    cursor_line <= gtk_source_gutter_lines_get_last (gutter->lines))
 		{
+			GdkRGBA highlight;
 			gint y;
 			gint height;
 
@@ -761,17 +751,14 @@ gtk_source_gutter_snapshot (GtkWidget   *widget,
 			                                         &y,
 			                                         &height);
 
-			gtk_style_context_save (style_context);
-			gtk_style_context_add_class (style_context, "current-line-number");
+			if (_gtk_source_view_get_current_line_number_background (gutter->view, &highlight))
+			{
+				int width = gtk_widget_get_width (widget);
 
-			gtk_snapshot_render_background (snapshot,
-							style_context,
-							0,
-							y,
-							gtk_widget_get_width (widget),
-							height);
-
-			gtk_style_context_restore (style_context);
+				gtk_snapshot_append_color (snapshot,
+				                           &highlight,
+				                           &GRAPHENE_RECT_INIT (0, y, width, height));
+			}
 		}
 	}
 
