@@ -22,13 +22,12 @@
 #include "gtksourcegutterrenderertext.h"
 
 /**
- * SECTION:gutterrenderertext
- * @Short_description: Renders text in the gutter
- * @Title: GtkSourceGutterRendererText
- * @See_also: #GtkSourceGutterRenderer, #GtkSourceGutter
+ * GtkSourceGutterRendererText:
+ * 
+ * Renders text in the gutter.
  *
- * A #GtkSourceGutterRendererText can be used to render text in a cell of
- * #GtkSourceGutter.
+ * A `GtkSourceGutterRendererText` can be used to render text in a cell of
+ * [class@Gutter].
  */
 
 typedef struct
@@ -151,8 +150,8 @@ gtk_source_gutter_renderer_text_snapshot_line (GtkSourceGutterRenderer *renderer
 
 	gtk_snapshot_render_layout (snapshot,
 	                            gtk_widget_get_style_context (GTK_WIDGET (text)),
-	                            x,
-	                            y,
+	                            ceilf (x),
+	                            ceilf (y),
 	                            layout);
 
 	if (priv->is_markup)
