@@ -44,7 +44,9 @@
 #include "gtksourcestyleschemechooserwidget.h"
 #include "gtksourcestyleschememanager-private.h"
 #include "gtksourcestyleschememanager-private.h"
+#include "gtksourcestyleschemepreview.h"
 #include "gtksourceview.h"
+#include "gtksourcevimimcontext.h"
 
 #ifdef G_OS_WIN32
 #define WIN32_LEAN_AND_MEAN
@@ -158,8 +160,6 @@ get_locale_dir (void)
  *
  * This function can be called several times, but is meant to be called at the
  * beginning of main(), before any other GtkSourceView function call.
- *
- * Since: 4.0
  */
 void
 gtk_source_init (void)
@@ -200,7 +200,9 @@ gtk_source_init (void)
 		g_type_ensure (GTK_SOURCE_TYPE_STYLE_SCHEME_CHOOSER);
 		g_type_ensure (GTK_SOURCE_TYPE_STYLE_SCHEME_CHOOSER_BUTTON);
 		g_type_ensure (GTK_SOURCE_TYPE_STYLE_SCHEME_CHOOSER_WIDGET);
+		g_type_ensure (GTK_SOURCE_TYPE_STYLE_SCHEME_PREVIEW);
 		g_type_ensure (GTK_SOURCE_TYPE_VIEW);
+		g_type_ensure (GTK_SOURCE_TYPE_VIM_IM_CONTEXT);
 
 		display = gdk_display_get_default ();
 
@@ -236,8 +238,6 @@ gtk_source_init (void)
  * It is not mandatory to call this function, it's just to be friendlier to
  * memory debugging tools. This function is meant to be called at the end of
  * main(). It can be called several times.
- *
- * Since: 4.0
  */
 
 /* Another way is to use a DSO destructor, see gconstructor.h in GLib.
