@@ -32,22 +32,21 @@
 #include "gtksourcelanguage-private.h"
 
 /**
- * SECTION:stylescheme
- * @Short_description: Controls the appearance of GtkSourceView
- * @Title: GtkSourceStyleScheme
- * @See_also: #GtkSourceStyle, #GtkSourceStyleSchemeManager
+ * GtkSourceStyleScheme:
+ * 
+ * Controls the appearance of [class@View].
  *
  * #GtkSourceStyleScheme contains all the text styles to be used in
- * #GtkSourceView and #GtkSourceBuffer. For instance, it contains text styles
+ * [class@View] and [class@Buffer]. For instance, it contains text styles
  * for syntax highlighting, it may contain foreground and background color for
  * non-highlighted text, color for the line numbers, current line highlighting,
  * bracket matching, etc.
  *
  * Style schemes are stored in XML files. The format of a scheme file is
- * documented in the [style scheme reference][style-reference].
+ * documented in the [style scheme reference](./style-reference.html).
  *
  * The two style schemes with IDs "classic" and "tango" follow more closely the
- * GTK+ theme (for example for the background color).
+ * GTK theme (for example for the background color).
  */
 
 #define STYLE_TEXT			"text"
@@ -59,10 +58,12 @@
 #define STYLE_SECONDARY_CURSOR		"secondary-cursor"
 #define STYLE_CURRENT_LINE		"current-line"
 #define STYLE_LINE_NUMBERS		"line-numbers"
+#define STYLE_LINE_NUMBERS_BORDER	"line-numbers-border"
 #define STYLE_CURRENT_LINE_NUMBER	"current-line-number"
 #define STYLE_RIGHT_MARGIN		"right-margin"
 #define STYLE_DRAW_SPACES		"draw-spaces"
 #define STYLE_SNIPPET_FOCUS		"snippet-focus"
+#define STYLE_BACKGROUND		"background"
 #define STYLE_BACKGROUND_PATTERN	"background-pattern"
 
 #define STYLE_SCHEME_VERSION		"1.0"
@@ -92,6 +93,7 @@ struct _GtkSourceStyleScheme
 	GHashTable *defined_styles;
 	GHashTable *style_cache;
 	GHashTable *named_colors;
+	GHashTable *metadata;
 
 	GtkCssProvider *css_provider;
 };
@@ -103,23 +105,10 @@ gtk_source_style_scheme_dispose (GObject *object)
 {
 	GtkSourceStyleScheme *scheme = GTK_SOURCE_STYLE_SCHEME (object);
 
-	if (scheme->named_colors != NULL)
-	{
-		g_hash_table_unref (scheme->named_colors);
-		scheme->named_colors = NULL;
-	}
-
-	if (scheme->style_cache != NULL)
-	{
-		g_hash_table_unref (scheme->style_cache);
-		scheme->style_cache = NULL;
-	}
-
-	if (scheme->defined_styles != NULL)
-	{
-		g_hash_table_unref (scheme->defined_styles);
-		scheme->defined_styles = NULL;
-	}
+	g_clear_pointer (&scheme->named_colors, g_hash_table_unref);
+	g_clear_pointer (&scheme->style_cache, g_hash_table_unref);
+	g_clear_pointer (&scheme->defined_styles, g_hash_table_unref);
+	g_clear_pointer (&scheme->metadata, g_hash_table_unref);
 
 	g_clear_object (&scheme->parent);
 	g_clear_object (&scheme->css_provider);
@@ -213,7 +202,7 @@ gtk_source_style_scheme_class_init (GtkSourceStyleSchemeClass *klass)
 	 * GtkSourceStyleScheme:id:
 	 *
 	 * Style scheme id, a unique string used to identify the style scheme
-	 * in #GtkSourceStyleSchemeManager.
+	 * in [class@StyleSchemeManager].
 	 */
 	g_object_class_install_property (object_class,
 					 PROP_ID,
@@ -294,8 +283,6 @@ gtk_source_style_scheme_init (GtkSourceStyleScheme *scheme)
  * @scheme: a #GtkSourceStyleScheme.
  *
  * Returns: @scheme id.
- *
- * Since: 2.0
  */
 const gchar *
 gtk_source_style_scheme_get_id (GtkSourceStyleScheme *scheme)
@@ -311,8 +298,6 @@ gtk_source_style_scheme_get_id (GtkSourceStyleScheme *scheme)
  * @scheme: a #GtkSourceStyleScheme.
  *
  * Returns: @scheme name.
- *
- * Since: 2.0
  */
 const gchar *
 gtk_source_style_scheme_get_name (GtkSourceStyleScheme *scheme)
@@ -328,8 +313,6 @@ gtk_source_style_scheme_get_name (GtkSourceStyleScheme *scheme)
  * @scheme: a #GtkSourceStyleScheme.
  *
  * Returns: (nullable): @scheme description (if defined), or %NULL.
- *
- * Since: 2.0
  */
 const gchar *
 gtk_source_style_scheme_get_description (GtkSourceStyleScheme *scheme)
@@ -346,8 +329,6 @@ gtk_source_style_scheme_get_description (GtkSourceStyleScheme *scheme)
  * Returns: (nullable) (array zero-terminated=1) (transfer none): a
  * %NULL-terminated array containing the @scheme authors or %NULL if
  * no author is specified by the style scheme.
- *
- * Since: 2.0
  */
 const gchar * const *
 gtk_source_style_scheme_get_authors (GtkSourceStyleScheme *scheme)
@@ -368,8 +349,6 @@ gtk_source_style_scheme_get_authors (GtkSourceStyleScheme *scheme)
  *
  * Returns: (nullable): @scheme file name if the scheme was created
  * parsing a style scheme file or %NULL in the other cases.
- *
- * Since: 2.0
  */
 const gchar *
 gtk_source_style_scheme_get_filename (GtkSourceStyleScheme *scheme)
@@ -411,8 +390,6 @@ color_parse (const gchar *color,
  * Returned value is actual color string suitable for gdk_rgba_parse().
  * It may be @name or part of @name so copy it or something, if you need
  * it to stay around.
- *
- * Since: 2.0
  */
 static const gchar *
 get_color_by_name (GtkSourceStyleScheme *scheme,
@@ -542,8 +519,6 @@ gtk_source_style_scheme_get_style_internal (GtkSourceStyleScheme *scheme,
  * Returns: (nullable) (transfer none): style which corresponds to @style_id in
  * the @scheme, or %NULL when no style with this name found.  It is owned by
  * @scheme and may not be unref'ed.
- *
- * Since: 2.0
  */
 /*
  * It's a little weird because we have named colors: styles loaded from
@@ -679,6 +654,20 @@ _gtk_source_style_scheme_get_current_line_color (GtkSourceStyleScheme *scheme,
 	return get_color (style, FALSE, color);
 }
 
+gboolean
+_gtk_source_style_scheme_get_current_line_number_color (GtkSourceStyleScheme *scheme,
+                                                        GdkRGBA              *color)
+{
+	GtkSourceStyle *style;
+
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), FALSE);
+	g_return_val_if_fail (color != NULL, FALSE);
+
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_CURRENT_LINE_NUMBER);
+
+	return get_color (style, FALSE, color);
+}
+
 /*
  * Returns TRUE if the style for background-pattern-color is set in the scheme
  */
@@ -696,6 +685,34 @@ _gtk_source_style_scheme_get_background_pattern_color (GtkSourceStyleScheme *sch
 	return get_color (style, FALSE, color);
 }
 
+gboolean
+_gtk_source_style_scheme_get_background_color (GtkSourceStyleScheme *scheme,
+                                               GdkRGBA              *color)
+{
+	GtkSourceStyle *style;
+
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), FALSE);
+	g_return_val_if_fail (color != NULL, FALSE);
+
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_BACKGROUND);
+
+	return get_color (style, FALSE, color);
+}
+
+gboolean
+_gtk_source_style_scheme_get_text_color (GtkSourceStyleScheme *scheme,
+                                         GdkRGBA              *color)
+{
+	GtkSourceStyle *style;
+
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), FALSE);
+	g_return_val_if_fail (color != NULL, FALSE);
+
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_TEXT);
+
+	return get_color (style, TRUE, color);
+}
+
 static void
 apply_css_style_cursors (GtkSourceStyleScheme *scheme,
                          GString              *css)
@@ -760,8 +777,6 @@ apply_css_style_cursors (GtkSourceStyleScheme *scheme,
  * @widget: (nullable): a #GtkWidget to apply styles to.
  *
  * Sets style colors from @scheme to the @widget.
- *
- * Since: 2.0
  */
 void
 _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
@@ -789,8 +804,6 @@ _gtk_source_style_scheme_apply (GtkSourceStyleScheme *scheme,
  * @widget: (nullable): a (#GtkWidget to unapply styles to.
  *
  * Removes the styles from @scheme in the @widget.
- *
- * Since: 3.0
  */
 void
 _gtk_source_style_scheme_unapply (GtkSourceStyleScheme *scheme,
@@ -902,11 +915,22 @@ generate_css_style (GtkSourceStyleScheme *scheme)
 		append_css_style (final_style, style, "textview border gutter");
 	}
 
-	style = gtk_source_style_scheme_get_style (scheme, STYLE_CURRENT_LINE_NUMBER);
+	/* Add border for line-numbers-border if specified */
+	style = gtk_source_style_scheme_get_style (scheme, STYLE_LINE_NUMBERS_BORDER);
 	if (style != NULL)
 	{
-		append_css_style (final_style, style, "textview gutter.current-line-number");
-		append_css_style (final_style, style, "textview gutterrenderer.current-line-number");
+		GdkRGBA color;
+
+		if (get_color (style, FALSE, &color))
+		{
+			char *bg = gdk_rgba_to_string (&color);
+			g_string_append_printf (final_style,
+			                        "textview border.left gutter {\n"
+			                        "  border-right: 1px solid %s;\n"
+			                        "}\n",
+			                        bg);
+			g_free (bg);
+		}
 	}
 
 	apply_css_style_cursors (scheme, final_style);
@@ -1201,6 +1225,38 @@ parse_style_scheme_child (GtkSourceStyleScheme  *scheme,
 		scheme->description = g_strdup (_((char*) tmp));
 		xmlFree (tmp);
 	}
+	else if (strcmp ((char*) node->name, "metadata") == 0)
+	{
+		for (xmlNode *child = node->children; child != NULL; child = child->next)
+		{
+			if (strcmp ((char*) child->name, "property") == 0)
+			{
+				xmlChar *name;
+				xmlChar *content;
+
+				name = xmlGetProp (child, BAD_CAST "name");
+				content = xmlNodeGetContent (child);
+
+				if (name != NULL && content != NULL)
+				{
+					if (scheme->metadata == NULL)
+					{
+						scheme->metadata = g_hash_table_new_full (g_str_hash,
+						                                          g_str_equal,
+						                                          g_free,
+						                                          g_free);
+					}
+
+					g_hash_table_insert (scheme->metadata,
+					                     g_strdup ((char *)name),
+					                     g_strdup ((char *)content));
+				}
+
+				xmlFree (name);
+				xmlFree (content);
+			}
+		}
+	}
 	else
 	{
 		g_set_error (error, ERROR_QUARK, 0, "unknown node '%s'", node->name);
@@ -1282,8 +1338,6 @@ parse_style_scheme_element (GtkSourceStyleScheme  *scheme,
  *
  * Returns: (nullable): new #GtkSourceStyleScheme created from file,
  * or %NULL on error.
- *
- * Since: 2.0
  */
 GtkSourceStyleScheme *
 _gtk_source_style_scheme_new_from_file (const gchar *filename)
@@ -1361,8 +1415,6 @@ _gtk_source_style_scheme_new_from_file (const gchar *filename)
  * @scheme: a #GtkSourceStyleScheme.
  *
  * Returns: (nullable): parent style scheme id or %NULL.
- *
- * Since: 2.0
  */
 const gchar *
 _gtk_source_style_scheme_get_parent_id (GtkSourceStyleScheme *scheme)
@@ -1380,8 +1432,6 @@ _gtk_source_style_scheme_get_parent_id (GtkSourceStyleScheme *scheme)
  * Sets @parent_scheme as parent scheme for @scheme, @scheme will
  * look for styles in @parent_scheme if it doesn't have style set
  * for given name.
- *
- * Since: 2.0
  */
 void
 _gtk_source_style_scheme_set_parent (GtkSourceStyleScheme *scheme,
@@ -1414,8 +1464,6 @@ _gtk_source_style_scheme_set_parent (GtkSourceStyleScheme *scheme,
  *
  * Returns: default style scheme to be used when user didn't set
  * style scheme explicitly.
- *
- * Since: 2.0
  */
 GtkSourceStyleScheme *
 _gtk_source_style_scheme_get_default (void)
@@ -1427,3 +1475,31 @@ _gtk_source_style_scheme_get_default (void)
 	return gtk_source_style_scheme_manager_get_scheme (manager,
 							   DEFAULT_STYLE_SCHEME);
 }
+
+/**
+ * gtk_source_style_scheme_get_metadata:
+ * @scheme: a #GtkSourceStyleScheme.
+ * @name: metadata property name.
+ *
+ * Gets a metadata property from the style scheme.
+ *
+ * Returns: (nullable) (transfer none): value of property @name stored in
+ *   the metadata of @scheme or %NULL if @scheme does not contain the
+ *   specified metadata property.
+ *
+ * Since: 5.4
+ */
+const char *
+gtk_source_style_scheme_get_metadata (GtkSourceStyleScheme *scheme,
+                                      const char           *name)
+{
+	g_return_val_if_fail (GTK_SOURCE_IS_STYLE_SCHEME (scheme), NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+
+	if (scheme->metadata == NULL)
+	{
+		return NULL;
+	}
+
+	return g_hash_table_lookup (scheme->metadata, name);
+}
