@@ -41,17 +41,16 @@
  * gtk_source_utils_unescape_search_text:
  * @text: the text to unescape.
  *
- * Use this function before gtk_source_search_settings_set_search_text(), to
+ * Use this function before [method@SearchSettings.set_search_text], to
  * unescape the following sequences of characters: `\n`, `\r`, `\t` and `\\`.
  * The purpose is to easily write those characters in a search entry.
  *
  * Note that unescaping the search text is not needed for regular expression
  * searches.
  *
- * See also: gtk_source_utils_escape_search_text().
+ * See also: [func@utils_escape_search_text].
  *
  * Returns: the unescaped @text.
- * Since: 3.10
  */
 gchar *
 gtk_source_utils_unescape_search_text (const gchar *text)
@@ -141,7 +140,7 @@ gtk_source_utils_unescape_search_text (const gchar *text)
  * search entry. The selection can contain tabulations, newlines, etc. So it's
  * better to escape those special characters to better fit in the search entry.
  *
- * See also: gtk_source_utils_unescape_search_text().
+ * See also: [func@utils_unescape_search_text].
  *
  * <warning>
  * Warning: the escape and unescape functions are not reciprocal! For example,
@@ -151,7 +150,6 @@ gtk_source_utils_unescape_search_text (const gchar *text)
  * </warning>
  *
  * Returns: the escaped @text.
- * Since: 3.10
  */
 gchar *
 gtk_source_utils_escape_search_text (const gchar* text)
@@ -391,6 +389,28 @@ _gtk_source_utils_pango_font_description_to_css (const PangoFontDescription *fon
 				ADD_KEYVAL (FONT_VARIANT, "small-caps");
 				break;
 
+#if PANGO_VERSION_CHECK(1, 49, 3)
+			case PANGO_VARIANT_ALL_SMALL_CAPS:
+				ADD_KEYVAL (FONT_VARIANT, "all-small-caps");
+				break;
+
+			case PANGO_VARIANT_PETITE_CAPS:
+				ADD_KEYVAL (FONT_VARIANT, "petite-caps");
+				break;
+
+			case PANGO_VARIANT_ALL_PETITE_CAPS:
+				ADD_KEYVAL (FONT_VARIANT, "all-petite-caps");
+				break;
+
+			case PANGO_VARIANT_UNICASE:
+				ADD_KEYVAL (FONT_VARIANT, "unicase");
+				break;
+
+			case PANGO_VARIANT_TITLE_CAPS:
+				ADD_KEYVAL (FONT_VARIANT, "titling-caps");
+				break;
+#endif
+
 			default:
 				break;
 		}
@@ -652,6 +672,7 @@ _gtk_source_view_jump_to_iter (GtkTextView       *text_view,
   int screen_yoffset;
   int current_x_scroll;
   int current_y_scroll;
+  int top_margin;
 
   g_return_if_fail (GTK_IS_TEXT_VIEW (text_view));
   g_return_if_fail (iter != NULL);
@@ -659,6 +680,8 @@ _gtk_source_view_jump_to_iter (GtkTextView       *text_view,
   g_return_if_fail (xalign >= 0.0 && xalign <= 1.0);
   g_return_if_fail (yalign >= 0.0 && yalign <= 1.0);
 
+  g_object_get (text_view, "top-margin", &top_margin, NULL);
+
   hadj = gtk_scrollable_get_hadjustment (GTK_SCROLLABLE (text_view));
   vadj = gtk_scrollable_get_vadjustment (GTK_SCROLLABLE (text_view));
 
@@ -753,5 +776,5 @@ _gtk_source_view_jump_to_iter (GtkTextView       *text_view,
   xvalue += current_x_scroll;
 
   gtk_adjustment_set_value (hadj, xvalue);
-  gtk_adjustment_set_value (vadj, yvalue);
+  gtk_adjustment_set_value (vadj, yvalue + top_margin);
 }
